@@ -4,11 +4,12 @@
 # plugininterface.py, provides an interface class for plugin extension modules
 #
 # TreeLine, an information storage program
-# Copyright (C) 2005, Douglas W. Bell
+# Copyright (C) 2006, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
-# terms of the GNU General Public License, Version 2.  This program is
-# distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
+# terms of the GNU General Public License, either Version 2 or any later
+# version.  This program is distributed in the hope that it will be useful,
+# but WITTHOUT ANY WARRANTY.  See the included LICENSE file for details.
 #*****************************************************************************
 
 
@@ -41,15 +42,23 @@
    operations.  Probably the most useful is the view update callback, which is
    called any time something changes in TreeLine requiring view or control
    availability updates.
+
+   Plugins used with windows binary installations are limited to the Python
+   modules that are used somewhere in TreeLine itself.  No other modules are
+   available, with the exception of urllib2, which is specifically included in
+   the binary for plugin use.
 """
 
-from treedoc import ReadFileError, PasswordError
-from nodeformat import NodeFormat
+import sys
+import os.path
+from PyQt4 import QtCore, QtGui
+import treedoc
+import nodeformat
+import optiondefaults
 import globalref
-from qt import QPixmap
 
 
-class PluginInterface:
+class PluginInterface(object):
     """Defines the available interface for the plugins"""
     def __init__(self, mainWin):
         self.mainWin = mainWin   # may be used in plugin as widget parent only
@@ -59,6 +68,15 @@ class PluginInterface:
         self.fileOpenCallbacks = []    # internal use only
         self.fileSaveCallbacks = []    # internal use only
 
+    def pluginPath(self):
+        """Return path of this plugin's directory"""
+        try:
+            frame = sys._getframe(1)
+            fileName = frame.f_code.co_filename
+        finally:
+            del frame
+        return os.path.dirname(fileName)
+
     #*************************************************************************
     #  Node Interfaces:
     #*************************************************************************
@@ -69,7 +87,7 @@ class PluginInterface:
 
     def getSelectedNodes(self):
         """Return a list of currently selected nodes"""
-        return globalref.docRef.selection[:]
+        return list(globalref.docRef.selection)
 
     def changeSelection(self, newSelectList):
         """Change tree selections, last item in newSelectList becomes the
@@ -140,20 +158,19 @@ class PluginInterface:
 
     def getFieldOutput(self, node, fieldName):
         """Return formatted text for the given fieldName data"""
-        field = node.nodeFormat.findField(fieldName)
+        field = node.nodeFormat().findField(fieldName)
         if field:
-            return node.nodeFormat.fieldText(field, node)
+            return node.nodeFormat().fieldText(field, node)
         return ''
 
     def getNodeFormatName(self, node):
         """Return the format type name for the given node"""
-        return node.nodeFormat.name
+        return node.formatName
 
     def setNodeFormat(self, node, formatName):
         """Set the given node to the given node format type"""
-        newFormat = globalref.docRef.treeFormats.findFormat(formatName)
-        if newFormat:
-            node.nodeFormat = newFormat
+        if formatName in globalref.docRef.treeFormats:
+            node.formatName = formatName
 
     def setDataChangeCallback(self, callbackFunc):
         """Set callback function to be called every time a node's dictionary
@@ -173,8 +190,8 @@ class PluginInterface:
         """Create a new node format, names must only contain characters
            [a-zA-Z0-9_.-].  If defaultFieldName, a text field is created
            and added to the title line and the first output line"""
-        globalref.docRef.treeFormats.append(NodeFormat(formatName, {}, \
-                                            defaultFieldName))
+        format = nodeformat.NodeFormat(formatName, {}, defaultFieldName)
+        globalref.docRef.treeFormats[formatName] = format
 
     def copyFileFormat(self, fileRef, password=''):
         """Copy the configuration from another TreeLine file,
@@ -185,161 +202,197 @@ class PluginInterface:
         try:
             globalref.docRef.treeFormats.configCopy(fileRef, password)
             return True
-        except (PasswordError, IOError, UnicodeError, ReadFileError):
+        except (treedoc.PasswordError, IOError, UnicodeError,
+                treedoc.ReadFileError):
             return False
 
     def getFormatIconName(self, formatName):
         """Return the node format's currently set icon name,
            a default setting will return an empty string,
            blank will return 'NoIcon'"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            return format.iconName
-        return ''
+        try:
+            return globalref.docRef.treeFormats[formatName].iconName
+        except KeyError:
+            return ''
 
     def setFormatIconName(self, formatName, iconName):
         """Set the node format's icon to iconName,
            an empty string or unknown icon name will get the default icon,
            use 'NoIcon' to get a blank"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            format.iconName = iconName
+        try:
+            globalref.docRef.treeFormats[formatName].iconName = iconName
+        except KeyError:
+            pass
 
-    def addTreeIcon(self, name, icon):
+    def addTreeIcon(self, name, image):
         """Add an icon to those available for use in the tree,
            icon data can be in any image format supported by Qt,
            if name matches one already loaded, the earlier one is replaced"""
-        globalref.treeIcons[name] = QPixmap(icon)
+        icon = QtGui.QIcon()
+        pixmap = QtGui.QPixmap(image)
+        if not pixmap.isNull():
+            icon.addPixmap(pixmap)
+            globalref.treeIcons[name] = icon
 
     def getTitleLineFormat(self, formatName):
         """Return the node format's title formatting line"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            return format.getLines()[0]
-        return ''
+        try:
+            return globalref.docRef.treeFormats[formatName].getLines()[0]
+        except KeyError:
+            return ''
 
     def setTitleLineFormat(self, formatName, newLine):
         """Set the node format's title formatting line to newLine"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            format.insertLine(newLine, 0)
+        try:
+            globalref.docRef.treeFormats[formatName].insertLine(newLine, 0)
+        except KeyError:
+            pass
 
     def getOutputFormatLines(self, formatName):
         """Return a list of the node format's output formatting lines"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            return format.getLines()[1:]
-        return []
+        try:
+            return globalref.docRef.treeFormats[formatName].getLines()[1:]
+        except KeyError:
+            return []
 
     def setOutputFormatLines(self, formatName, lineList):
         """Set the node format's output formatting lines to lineList"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            format.lineList = format.lineList[:1]
-            for line in lineList:
-                format.addLine(line)
+        try:
+            format = globalref.docRef.treeFormats[formatName]
+        except KeyError:
+            return
+        format.lineList = format.lineList[:1]
+        for line in lineList:
+            format.addLine(line)
 
     def getFormatFieldNames(self, formatName):
         """Return a list of the node format's field names"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            return format.fieldNames()
-        return []
+        try:
+            return globalref.docRef.treeFormats[formatName].fieldNames()
+        except KeyError:
+            return []
 
     def addNewFormatField(self, formatName, fieldName, fieldType='Text'):
         """Add a new field to the node format, type should be one of:
            Text, Number, Choice, Combination, AutoChoice, Date, Time, 
            Boolean, URL, Path, Email, InternalLink, ExecuteLink, Picture"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            format.addNewField(fieldName, {'type': fieldType})
+        try:
+            globalref.docRef.treeFormats[formatName].\
+                             addNewField(fieldName, {'type': fieldType})
+        except KeyError:
+            pass
 
     def getFormatFieldType(self, formatName, fieldName):
         """Return the type of the given field in the given format"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                return field.typeName
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return ''
+        if field:
+            return field.typeName
+        return ''
 
     def changeFormatFieldType(self, formatName, fieldName, newFieldType):
         """Change the type of the given field in the given format,
            type should be one of:  Text, Number, Choice,
            Combination, AutoChoice, Date, Time, Boolean, URL, Path, Email,
            InternalLink, ExecuteLink, Picture"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                field.changeType(newFieldType)
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return
+        if field:
+            field.changeType(newFieldType)
 
     def getFormatFieldFormat(self, formatName, fieldName):
         """Return the format code string of the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                return field.format
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return ''
+        if field:
+            return field.format
+        return ''
 
     def setFormatFieldFormat(self, formatName, fieldName, newFieldFormat):
         """Change the format code string of the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                field.format = newFieldFormat
-                field.initFormat()
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return
+        if field:
+            field.format = newFieldFormat
+            field.initFormat()
 
     def getFormatFieldExtraText(self, formatName, fieldName):
         """Return a tuple of the prefix and suffix text of the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                return (field.prefix, field.suffix)
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return ('', '')
+        if field:
+            return (field.prefix, field.suffix)
+        return ('', '')
 
     def setFormatFieldExtraText(self, formatName, fieldName, newPrefix='',
                                 newSuffix=''):
         """Set the format prefix and suffix text of the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                field.prefix = newPrefix
-                field.suffix = newSuffix
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return
+        if field:
+            field.prefix = newPrefix
+            field.suffix = newSuffix
 
     def getFormatFieldHtmlProp(self, formatName, fieldName):
         """Return True if the given field is set to use HTML,
            False for plain text"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                return field.html
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return False
+        if field:
+            return field.html
+        return False
 
     def setFormatFieldHtmlProp(self, formatName, fieldName, htmlProp=True):
         """Change the HTML handling of the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                field.html = htmlProp
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return
+        if field:
+            field.html = htmlProp
 
     def getFormatFieldNumLines(self, formatName, fieldName):
         """Return the number of lines set for the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                return field.numLines
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return 0
+        if field:
+            return field.numLines
+        return 0
 
     def setFormatFieldNumLines(self, formatName, fieldName, numLines):
         """Set the number of lines set for the given field"""
-        format = globalref.docRef.treeFormats.findFormat(formatName)
-        if format:
-            field = format.findField(fieldName)
-            if field:
-                field.numLines = numLines
+        try:
+            field = globalref.docRef.treeFormats[formatName].\
+                                     findField(fieldName)
+        except KeyError:
+            return
+        if field:
+            field.numLines = numLines
 
     #*************************************************************************
     #  View Interfaces:
@@ -356,6 +409,20 @@ class PluginInterface:
            but is a good way to check for specific changes)"""
         self.viewUpdateCallbacks.append(callbackFunc)
 
+    def getActiveEditView(self):
+        """Return the currently active text editor in the Data Editor
+           right-hand view.  This does not include the combo boxes used for
+           some fields.  Returns None if something else has the focus."""
+        return self.mainWin.focusWidgetWithAttr('addHtmlTag')
+
+    def insertEditViewText(self, text):
+        """Inserts the given text into the currently active text editor
+           in the Data Editor right-hand view
+           (if one of the editors has the focus"""
+        editor = self.getActiveEditView()
+        if editor:
+            editor.insertPlainText(text)
+
     #*************************************************************************
     #  File Interfaces:
     #*************************************************************************
@@ -379,14 +446,14 @@ class PluginInterface:
            returns True/False on success/failure"""
         if password:
             if hasattr(fileRef, 'read'):
-                fileName = unicode(fileRef.name, globalref.docRef.localEncoding)
+                fileName = unicode(fileRef.name, sys.getfilesystemencoding())
             else:
                 fileName = fileRef
             globalref.docRef.setPassword(fileName, password)
         try:
             globalref.docRef.readFile(fileRef)
             return True
-        except (PasswordError, IOError, UnicodeError):
+        except (treedoc.PasswordError, IOError, UnicodeError):
             return False
 
     def saveFile(self, fileRef):
@@ -402,7 +469,7 @@ class PluginInterface:
            returns True/False on success/failure"""
         if password:
             if hasattr(fileRef, 'read'):
-                fileName = unicode(fileRef.name, globalref.docRef.localEncoding)
+                fileName = unicode(fileRef.name, sys.getfilesystemencoding())
             else:
                 fileName = fileRef
             globalref.docRef.setPassword(fileName, password)
@@ -412,6 +479,10 @@ class PluginInterface:
         except IOError:
             return False
 
+    def getCurrentFileName(self):
+        """Return the currently open filename"""
+        return globalref.docRef.fileName
+
     def getDocModified(self):
         """Return True if the current document is marked as modified,
            False otherwise"""
@@ -435,7 +506,7 @@ class PluginInterface:
         """Set callback function to be called after a file is saved"""
         self.fileSaveCallbacks.append(callbackFunc)
 
-    def exportHtml(self, fileRef, includeRoot=True, openOnly=False, indent=20, \
+    def exportHtml(self, fileRef, includeRoot=True, openOnly=False, indent=20,
                    addHeader=False):
         """Export current branch to single-column HTML,
            fileRef is either a file path string or a file-like object
@@ -443,7 +514,7 @@ class PluginInterface:
            remaining parameters are options,
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportHtml(fileRef, self.getCurrentNode(), \
+            globalref.docRef.exportHtml(fileRef, self.getCurrentNode(),
                                         includeRoot, openOnly, indent, 
                                         addHeader)
             return True
@@ -468,7 +539,7 @@ class PluginInterface:
            (if it is a file-like object, fileRef.name must be defined)
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportTrlSubtree(fileRef, self.getCurrentNode())
+            globalref.docRef.exportTrlSubtree(fileRef, [self.getCurrentNode()])
             return True
         except IOError:
             return False
@@ -479,7 +550,7 @@ class PluginInterface:
            (if it is a file-like object, fileRef.name must be defined)
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportTable(fileRef, self.getCurrentNode())
+            globalref.docRef.exportTable(fileRef, [self.getCurrentNode()])
             return True
         except IOError:
             return False
@@ -490,9 +561,9 @@ class PluginInterface:
            (if it is a file-like object, fileRef.name must be defined)
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportTabbedTitles(fileRef, \
-                                                   self.getCurrentNode(), \
-                                                   includeRoot, openOnly)
+            globalref.docRef.exportTabbedTitles(fileRef,
+                                                [self.getCurrentNode()],
+                                                True, includeRoot, openOnly)
             return True
         except IOError:
             return False
@@ -503,7 +574,7 @@ class PluginInterface:
            (if it is a file-like object, fileRef.name must be defined)
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportXbel(fileRef, self.getCurrentNode())
+            globalref.docRef.exportXbel(fileRef, [self.getCurrentNode()])
             return True
         except IOError:
             return False
@@ -514,7 +585,8 @@ class PluginInterface:
            (if it is a file-like object, fileRef.name must be defined)
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportHtmlBookmarks(fileRef, self.getCurrentNode())
+            globalref.docRef.exportHtmlBookmarks(fileRef,
+                                                 [self.getCurrentNode()])
             return True
         except IOError:
             return False
@@ -525,7 +597,7 @@ class PluginInterface:
            (if it is a file-like object, fileRef.name must be defined)
            returns True on success, False on failure"""
         try:
-            globalref.docRef.exportGenericXml(fileRef, self.getCurrentNode())
+            globalref.docRef.exportGenericXml(fileRef, [self.getCurrentNode()])
             return True
         except IOError:
             return False
@@ -539,13 +611,43 @@ class PluginInterface:
         return self.mainWin.menuBar()
 
     def getPulldownMenu(self, index):
-        """Return top pulldown menu at position index (QPopupMenu),
+        """Return top pulldown menu at position index (QMenu),
            return None if index is not valid"""
         try:
             return self.mainWin.pulldownMenuList[index]
         except IndexError:
             return None
 
+    def addMenuAction(self, name, action, initKey=''):
+        """Adds a QAction to the shortcut key editor (menu section) and to
+           the toolbar editor (if it has an icon).  This does not add it to a
+           menu (use menu.insertAction(...) on a menu from functions above).
+           Give a name without punctuation or spaces (spaces are added in the
+           editor based on CamelCase splits); initKey is optional control
+           key string"""
+        optiondefaults.menuKeyBindList.append((name, initKey))
+        optiondefaults.cmdTranslationDict[name] = name
+        globalref.options.addDefaultKey(name, initKey)
+        self.mainWin.actions[name] = action
+        icon = action.icon()
+        if icon and not icon.isNull():
+            self.mainWin.toolIcons[name.lower()] = icon
+        self.mainWin.setupShortcuts()
+        self.mainWin.setupToolbars()
+
+    def addShortcutKey(self, name, function, initKey=''):
+        """Adds a non-menu keyboard shortcut that the user can set in the
+           Shortcut Editor.  Give a name without punctuation or spaces (spaces
+           are added in the editor based on CamelCase splits), function is
+           any Python function, initKey is optional control key string"""
+        optiondefaults.otherKeyBindList.append((name, initKey))
+        optiondefaults.cmdTranslationDict[name] = name
+        globalref.options.addDefaultKey(name, initKey)
+        shortcut = QtGui.QShortcut(QtGui.QKeySequence(), self.mainWin,
+                                   function)
+        self.mainWin.shortcuts[name] = shortcut
+        self.mainWin.setupShortcuts()
+
     #*************************************************************************
     #  Internal methods (not for plugin use):
     #*************************************************************************
